@@ -684,6 +684,10 @@ void PackHttpRequest(butil::IOBuf* buf,
     // may not echo back this field. But we send it anyway.
     accessor.get_sending_socket()->set_correlation_id(correlation_id);
 
+    // Store http request method into Socket since http response parser needs it,
+    // and skips response body if request method is HEAD.
+    accessor.get_sending_socket()->set_http_request_method(header->method());
+
     MakeRawHttpRequest(buf, header, cntl->remote_side(),
                        &cntl->request_attachment());
     if (FLAGS_http_verbose) {
@@ -934,7 +938,12 @@ HttpResponseSender::~HttpResponseSender() {
         }
     } else {
         butil::IOBuf* content = NULL;
-        if (cntl->Failed() || !cntl->has_progressive_writer()) {
+        if ((cntl->Failed() || !cntl->has_progressive_writer()) &&
+            // https://datatracker.ietf.org/doc/html/rfc7231#section-4.3.2
+            // The HEAD method is identical to GET except that the server MUST NOT
+            // send a message body in the response (i.e., the response terminates at
+            // the end of the header section).
+            req_header->method() != HTTP_METHOD_HEAD) {
             content = &cntl->response_attachment();
         }
         butil::IOBuf res_buf;
@@ -1097,7 +1106,9 @@ ParseResult ParseHttpMessage(butil::IOBuf *source, Socket *socket,
             //    source is likely to be empty.
             return MakeParseError(PARSE_ERROR_NOT_ENOUGH_DATA);
         }
-        http_imsg = new (std::nothrow) HttpContext(socket->is_read_progressive());
+        http_imsg = new (std::nothrow) HttpContext(
+            socket->is_read_progressive(),
+            socket->http_request_method());
         if (http_imsg == NULL) {
             LOG(FATAL) << "Fail to new HttpContext";
             return MakeParseError(PARSE_ERROR_NO_RESOURCE);
@@ -1477,30 +1488,28 @@ void ProcessHttpRequest(InputMessageBase *msg) {
             const HttpContentType content_type =
                 ParseContentType(req_header.content_type(), &is_grpc_ct);
             const std::string* encoding = NULL;
-            if (is_http2) {
-                if (is_grpc_ct) {
-                    bool grpc_compressed = false;
-                    if (!RemoveGrpcPrefix(&req_body, &grpc_compressed)) {
-                        cntl->SetFailed(EREQUEST, "Invalid gRPC request");
+            if (is_http2 && is_grpc_ct) {
+                bool grpc_compressed = false;
+                if (!RemoveGrpcPrefix(&req_body, &grpc_compressed)) {
+                    cntl->SetFailed(EREQUEST, "Invalid gRPC request");
+                    return;
+                }
+                if (grpc_compressed) {
+                    encoding = req_header.GetHeader(common->GRPC_ENCODING);
+                    if (encoding == NULL) {
+                        cntl->SetFailed(
+                            EREQUEST, "Fail to find header `grpc-encoding'"
+                            " in compressed gRPC request");
                         return;
                     }
-                    if (grpc_compressed) {
-                        encoding = req_header.GetHeader(common->GRPC_ENCODING);
-                        if (encoding == NULL) {
-                            cntl->SetFailed(
-                                EREQUEST, "Fail to find header `grpc-encoding'"
-                                " in compressed gRPC request");
-                            return;
-                        }
-                    }
-                    int64_t timeout_value_us =
-                        ConvertGrpcTimeoutToUS(req_header.GetHeader(common->GRPC_TIMEOUT));
-                    if (timeout_value_us >= 0) {
-                        accessor.set_deadline_us(
-                                butil::gettimeofday_us() + timeout_value_us);
-                    }
                 }
-            } else {
+                int64_t timeout_value_us =
+                    ConvertGrpcTimeoutToUS(req_header.GetHeader(common->GRPC_TIMEOUT));
+                if (timeout_value_us >= 0) {
+                    accessor.set_deadline_us(
+                            butil::gettimeofday_us() + timeout_value_us);
+                }
+            } else { // http or h2 but not grpc
                 encoding = req_header.GetHeader(common->CONTENT_ENCODING);
             }
             if (encoding != NULL && *encoding == common->GZIP) {
